//
//  OpenGL Ex20opengl Widget
//

#ifndef EX20OPENGL_H
#define EX20OPENGL_H

#include <QtOpenGL>
#include <QString>
#include <QVector>

//  Star Structure
typedef struct
{
   float x,y,z;  //  Position
   float u,v,w;  //  Velocity
   float r,g,b;  //  Color
}  Star;

class Ex20opengl : public QGLWidget
{
Q_OBJECT
private:
   bool    init;      //  Initialized
   bool    move;      //  Light animation
   float   zh;        //  Light position
   float   x0,y0;     //  Object center
   float   z0;        //  Zoom
   int     N;         //  Star count
   int     src,dst;   //  Source and destination
   int     mode;      //  Display mode
   int     th,ph;     //  Display angles
   bool    mouse;     //  Mouse pressed
   QPoint  pos;       //  Mouse position
   int     fov;       //  Field of view
   double  dim;       //  Display size
   float   vel;       //  Initial velocity
   int     fps;       //  Frames per second
   int     sec0;      //  Seconds
   double  asp;       //  Sceen aspect ratio
   Star*   stars;     //  Vertices

   QGLShaderProgram shader; //  Shaders
   QTimer           timer;  //  Timer for animations
   QElapsedTimer    time;   //  Track elapsed time
public:
   Ex20opengl(QWidget* parent=0);                  //  Constructor
   QSize sizeHint() const {return QSize(400,400);} //  Default size of widget
public slots:
    void setShader(int sel);               //  Slot to set shader
    void reset();                          //  Reset view
signals:
    void angles(QString text);             //  Signal for view angles
    void FPS(QString text);                //  Signal for fps
protected:
    void initializeGL();                   //  Initialize widget
    void resizeGL(int width, int height);  //  Resize widget
    void paintGL();                        //  Draw widget
    void mousePressEvent(QMouseEvent*);    //  Mouse pressed
    void mouseReleaseEvent(QMouseEvent*);  //  Mouse released
    void mouseMoveEvent(QMouseEvent*);     //  Mouse moved
    void wheelEvent(QWheelEvent*);         //  Mouse wheel
private:
   void Fatal(QString message);            //  Error handler
   void Projection();                      //  Update projection
   void Shader(QGLShaderProgram& shader,QString vert,QString geom,QString frag);  //  Create shader
   void Move(int k);
   void Step();
   void InitLoc();
};

#endif
